use gtk4 as gtk;

use std::{num::ParseFloatError, str::FromStr};

use gtk::{
    prelude::{TextBufferExt, TextViewExt},
    TextBuffer,
};

use crate::{model::models::SignalReduce, view::components::input::Input};

pub fn get_error_message(error_instance: ParseFloatError) -> Option<&'static str> {
    match error_instance.to_string().as_str() {
        "cannot parse float from empty string" => Some("Вы не ввели данные в поле/-я"),
        "invalid float literal" => Some("Вы ввели не корректные данные в поле/-я"),
        _ => None,
    }
}

pub fn parse_fields(all_inputs: Vec<Input>) -> Result<SignalReduce, ParseFloatError> {
    let mut values: [f64; 6] = [0f64; 6];

    for (i, input) in all_inputs.iter().enumerate() {
        let input_text_buffer: TextBuffer = input.clone().get_input().buffer();
        let try_extract_value = f64::from_str(
            input_text_buffer
                .text(
                    &input_text_buffer.start_iter(),
                    &input_text_buffer.end_iter(),
                    false,
                )
                .as_str()
                .trim(),
        );
        match try_extract_value {
            Ok(value) => values[i] = value,
            Err(error) => return Err(error),
        }
    }

    Ok(SignalReduce(
        values[0], values[1], values[2], values[3], values[4], values[5],
    ))
}
