use gtk4 as gtk;

use crate::{model::builder_traits::*, view::properties::*};
use gtk::{prelude::*, *};

pub type InputLabel = String;

#[derive(Clone)]
pub struct Input {
    component: Box,
    input: TextView,
}

pub struct InputBuilder {
    align: Alignment,
    label: InputLabel,
    margins: MarginData,
}

impl Product<InputBuilder, Box> for Input {
    fn builder() -> InputBuilder {
        InputBuilder {
            align: Alignment {
                horizontal: Align::Start,
                vertical: Align::Start,
            },
            label: String::new(),
            margins: MarginData::EqualsMargin(5),
        }
    }

    fn get(self) -> Box {
        self.component
    }
}

impl Input {
    pub fn get_input(self) -> TextView {
        self.input
    }
}

impl InputBuilder {
    pub fn set_label(mut self, label: &str) -> Self {
        self.label = String::from(label);

        self
    }

    pub fn set_align(mut self, align: Alignment) -> Self {
        self.align = align;

        self
    }

    pub fn set_margins(mut self, margin: MarginData) -> Self {
        self.margins = margin;

        self
    }

    pub fn build(self, monospace: bool, wrap_mode: WrapMode, input_height: i32) -> Input {
        let input_component = Box::new(Orientation::Vertical, 0);

        let input_label = Label::builder()
            .halign(self.align.horizontal)
            .valign(self.align.vertical)
            .set_margin(self.margins)
            .label(self.label)
            .build();

        let text_view_input = TextView::builder()
            .monospace(monospace)
            .height_request(input_height)
            .set_text_view_margin(MarginData::EqualsMargin(6))
            .wrap_mode(wrap_mode)
            .build();

        let text_view_input_frame = Frame::builder()
            .child(&text_view_input)
            .set_margin(MarginData::MultipleMargin((0, 5, 0, 5)))
            .build();

        input_component.append(&input_label);
        input_component.append(&text_view_input_frame);

        Input {
            component: input_component,
            input: text_view_input,
        }
    }
}
