use gtk4 as gtk;

use super::builder_traits::*;
use gtk::{Box, Label, Notebook};

pub type TabLabel = Label;
pub type TabContent = Box;
pub type TabPage<'a> = (&'a str, TabContent);

#[derive(Clone)]
pub struct TabsBuilder {
    tabs: Vec<(TabLabel, TabContent)>,
}

pub struct Tabs {
    tabs_wrapper: Notebook,
}

impl Product<TabsBuilder, Notebook> for Tabs {
    fn builder() -> TabsBuilder {
        TabsBuilder { tabs: Vec::new() }
    }

    fn get(self) -> Notebook {
        self.tabs_wrapper
    }
}

impl Builder<Tabs, TabPage<'_>, &str> for TabsBuilder {
    fn build(&self, build_param: &str) -> Tabs {
        let tabs_wrapper = Notebook::builder().group_name(build_param).build();

        self.tabs.iter().for_each(|(label, content)| {
            tabs_wrapper.append_page(content, Some(label));
        });

        Tabs { tabs_wrapper }
    }

    fn append_item(mut self, item: TabPage) -> Self {
        self.append_tab_private(item);

        self
    }

    fn append_items(mut self, items: Vec<TabPage>) -> Self {
        for item in items {
            self.append_tab_private(item);
        }

        self
    }
}

impl TabsBuilder {
    fn append_tab_private(&mut self, item: TabPage) {
        let tab_label = Label::new(Some(item.0));
        self.tabs.push((tab_label, item.1));
    }
}

